@@ -13,7 +13,7 @@ from scipy.interpolate import interp1d
 from shapely.geometry import box
 
 from .utils import timing, map_blocks_coords, BlockingActorProxy, merge_yaml, \
-    to_lon180, config
+    to_lon180, config, datetime
 from .sentinel1_meta import Sentinel1Meta
 from .ipython_backends import repr_mimebundle
 import datatree
@@ -40,6 +40,7 @@ mapping_dataset_geoloc = {'latitude': 'latitude',
                           'altitude': 'height',
                           'azimuth_time': 'azimuthTime',
                           'slant_range_time': 'slantRangeTime',
+                          'offboresight': 'offboresightAngle',
                           }
 
 
@@ -95,11 +96,10 @@ class Sentinel1Dataset(BaseDataset):
                  resampling=rasterio.enums.Resampling.rms,
                  luts=False, chunks={'line': 5000, 'sample': 5000},
                  dtypes=None, patch_variable=True, lazyloading=True,
-                 recalibration=False, aux_config_name='v_IPF_36'):
+                 recalibration=False):
         # default dtypes
         if dtypes is not None:
             self._dtypes.update(dtypes)
-
         # default meta for map_blocks output.
         # as asarray is imported from numpy, it's a numpy array.
         # but if later we decide to import asarray from cupy, il will be a cupy.array (gpu)
@@ -139,6 +139,16 @@ class Sentinel1Dataset(BaseDataset):
         # geoloc
         geoloc = self.sar_meta.geoloc
         geoloc.attrs['history'] = 'annotations'
+        geoloc["offboresightAngle"] = geoloc.elevationAngle - \
+            (30.1833947 * geoloc.latitude ** 0 + \
+            0.0082998714 * geoloc.latitude ** 1 - \
+            0.00031181534 * geoloc.latitude ** 2 - \
+            0.0943533e-07 * geoloc.latitude ** 3 + \
+            3.0191435e-08 * geoloc.latitude ** 4 + \
+            4.968415428e-12 *geoloc.latitude ** 5 - \
+            9.315371305e-13 * geoloc.latitude ** 6) + 29.45
+        geoloc["offboresightAngle"].attrs['comment']='built from elevation angle and latitude'
+        
         # bursts
         bu = self.sar_meta._bursts
         bu.attrs['history'] = 'annotations'
@@ -157,7 +167,6 @@ class Sentinel1Dataset(BaseDataset):
 
         # noise levels LUTs
         ds_noise_range = self.sar_meta.get_noise_range_raw
-        ds_noise_range.attrs['history'] = 'noise'
         ds_noise_azi = self.sar_meta.get_noise_azi_raw
 
         # antenna pattern
@@ -172,7 +181,6 @@ class Sentinel1Dataset(BaseDataset):
             #                                                 'noise_lut'])  # patch applied here is distinct to same patch applied on interpolated noise LUT
             ds_noise_range['noise_lut'] = self._patch_lut(ds_noise_range[
                 'noise_lut'])  # patch applied here is distinct to same patch applied on interpolated noise LUT
-        ds_noise_azi.attrs['history'] = 'noise'
 
         self.datatree = datatree.DataTree.from_dict({'measurement': DN_tmp, 'geolocation_annotation': geoloc,
                                                      'bursts': bu, 'FMrate': FM, 'doppler_estimate': dop,
@@ -187,8 +195,6 @@ class Sentinel1Dataset(BaseDataset):
                                                      })
 
         # apply recalibration ?
-
-        self.aux_config_name = aux_config_name
         self.apply_recalibration = recalibration
         if self.apply_recalibration and (self.sar_meta.swath != "EW" and self.sar_meta.swath != "IW"):
             self.apply_recalibration = False
@@ -285,15 +291,20 @@ class Sentinel1Dataset(BaseDataset):
         self.datatree.attrs.update(self.sar_meta.to_dict("all"))
 
         # load land_mask by default for GRD products
+
+
         if 'GRD' in str(self.datatree.attrs['product']):
             self.add_high_resolution_variables(
-                patch_variable=patch_variable, luts=luts, lazy_loading=lazyloading)
+                                   patch_variable=patch_variable, luts=luts, lazy_loading=lazyloading)
             if self.apply_recalibration:
                 self.select_gains()
             self.apply_calibration_and_denoising()
 
         # added 6 fev 23, to fill  empty attrs
         self.datatree['measurement'].attrs = self.datatree.attrs
+        if self.sar_meta.product == 'SLC' and 'WV' not in self.sar_meta.swath:  # TOPS cases
+            tmp = self.corrected_range_noise_lut(self.datatree)
+            self.datatree['noise_range'].ds = tmp # the corrcted noise_range dataset shold now contain an attrs 'corrected_range_noise_lut'
         self.sliced = False
         """True if dataset is a slice of original L1 dataset"""
 
@@ -303,6 +314,35 @@ class Sentinel1Dataset(BaseDataset):
         # save original bbox
         self._bbox_coords_ori = self._bbox_coords
 
+    def corrected_range_noise_lut(self,dt):
+        """
+        Patch F.Nouguier see https://jira-projects.cls.fr/browse/MPCS-3581 and https://github.com/umr-lops/xsar_slc/issues/175
+        Return range noise lut with corrected line numbering. This function should be used only on the full SLC dataset dt
+        Args:
+            dt (xarray.datatree) : datatree returned by xsar corresponding to one subswath
+        Return:
+            (xarray.dataset) : range noise lut with corrected line number
+        """
+        # Detection of azimuthTime jumps (linked to burst changes). Burst sensingTime should NOT be used since they have erroneous value too !
+        line_shift = 0
+        tt = dt['measurement']['time']
+        i_jump = np.ravel(np.argwhere(np.diff(tt)<np.timedelta64(0))+1) # index of jumps
+        line_jump_meas = dt['measurement']['line'][i_jump] # line number of jumps
+        # line_jump_noise = np.ravel(dt['noise_range']['line'][1:-1].data) # annotated line number of burst begining, this one is corrupted for some S1 TOPS product
+        line_jump_noise = np.ravel(dt['noise_range']['line'][1:1+len(line_jump_meas)].data) # annoted line number of burst begining
+        burst_first_lineshift = line_jump_meas-line_jump_noise
+        if len(np.unique(burst_first_lineshift))==1:
+            line_shift = int(np.unique(burst_first_lineshift)[0])
+            logging.debug('line_shift: %s',line_shift)
+        else:
+            raise ValueError('Inconsistency in line shifting : {}'.format(burst_first_lineshift))
+        res = dt['noise_range'].ds.assign_coords({'line':dt['noise_range']['line']+line_shift})
+        if line_shift==0:
+            res.attrs['corrected_range_noise_lut'] = 'no change'
+        else:
+            res.attrs['corrected_range_noise_lut'] = 'shift : %i lines'%line_shift
+        return res
+
     def select_gains(self):
         """
         attribution of the good swath gain by getting the swath number of each pixel 
@@ -460,7 +500,7 @@ class Sentinel1Dataset(BaseDataset):
             self._dataset = xr.merge(ds_merge_list)
             self._dataset.attrs = attrs
             geoloc_vars = ['altitude', 'azimuth_time', 'slant_range_time',
-                           'incidence', 'elevation', 'longitude', 'latitude']
+                           'incidence', 'elevation', 'longitude', 'latitude', 'offboresight']
 
             for vv in skip_variables:
                 if vv in geoloc_vars:
@@ -468,13 +508,27 @@ class Sentinel1Dataset(BaseDataset):
 
             self._dataset = self._dataset.merge(
                 self._load_from_geoloc(geoloc_vars, lazy_loading=lazy_loading))
-
-            
-            self.add_swath_number()   
+                         
+            if 'GRD' in str(self.datatree.attrs['product']):
+                self.add_swath_number()
                 
-            if self.apply_recalibration:
-                self.add_gains(config["auxiliary_names"][self.sar_meta.short_name.split(":")[-2][0:3]][self.aux_config_name]["AUX_CAL"],
-                               config["auxiliary_names"][self.sar_meta.short_name.split(":")[-2][0:3]][self.aux_config_name]["AUX_PP1"])
+                if self.apply_recalibration:
+                    path_dataframe_aux = config["path_dataframe_aux"]
+                    dataframe_aux = pd.read_csv(path_dataframe_aux)
+
+                    sel_cal = dataframe_aux.loc[(dataframe_aux.sat_name == self.sar_meta.manifest_attrs['satellite']) & 
+                                                (dataframe_aux.aux_type == "CAL") &
+                                                (dataframe_aux.validation_date <= self.sar_meta.start_date)]
+                    sel_cal = sel_cal.sort_values(by = ["validation_date","generation_date"],ascending=False)
+                    path_new_cal = sel_cal.iloc[0].aux_path
+
+                    sel_pp1 = dataframe_aux.loc[(dataframe_aux.sat_name == self.sar_meta.manifest_attrs['satellite']) & 
+                                (dataframe_aux.aux_type == "PP1") 
+                                & (dataframe_aux.validation_date <= self.sar_meta.start_date)]
+                    sel_pp1 = sel_pp1.sort_values(by = ["validation_date","generation_date"],ascending=False)
+                    path_new_pp1 = sel_pp1.iloc[0].aux_path
+
+                    self.add_gains(path_new_cal, path_new_pp1)
 
             rasters = self._load_rasters_vars()
             if rasters is not None:
@@ -526,14 +580,14 @@ class Sentinel1Dataset(BaseDataset):
                                 'line': self._dataset.coords["line"], 'sample': self._dataset.coords["sample"]})
 
         # Supposons que dataset.swaths ait 45 éléments comme mentionné
-        for i in range(len(self.datatree['swath_merging'].swaths)):
+        for i in range(len(self.datatree['swath_merging'].ds['swaths'])):
             y_min, y_max = self.datatree['swath_merging']['firstAzimuthLine'][
                 i], self.datatree['swath_merging']['lastAzimuthLine'][i]
             x_min, x_max = self.datatree['swath_merging']['firstRangeSample'][
                 i], self.datatree['swath_merging']['lastRangeSample'][i]
 
             # Localisation des pixels appartenant à ce swath
-            swath_index = int(self.datatree['swath_merging'].swaths[i])
+            swath_index = int(self.datatree['swath_merging'].ds['swaths'][i])
 
             condition = (self._dataset.line >= y_min) & (self._dataset.line <= y_max) & (
                 self._dataset.sample >= x_min) & (self._dataset.sample <= x_max)
@@ -563,11 +617,11 @@ class Sentinel1Dataset(BaseDataset):
         logger.debug(
             f"doing recalibration with AUX_CAL = {new_aux_cal_name} & AUX_PP1 = {new_aux_pp1_name}")
 
-        path_aux_cal_new = get_path_aux_cal(new_aux_cal_name)
+        path_aux_cal_new = get_path_aux_cal(os.path.basename(new_aux_cal_name))
         path_aux_cal_old = get_path_aux_cal(
             os.path.basename(self.sar_meta.manifest_attrs["aux_cal"]))
 
-        path_aux_pp1_new = get_path_aux_pp1(new_aux_pp1_name)
+        path_aux_pp1_new = get_path_aux_pp1(os.path.basename(new_aux_pp1_name))
         path_aux_pp1_old = get_path_aux_pp1(
             os.path.basename(self.sar_meta.manifest_attrs["aux_pp1"]))
 
@@ -680,7 +734,11 @@ class Sentinel1Dataset(BaseDataset):
 
         self.datatree['recalibration'] = self.datatree['recalibration'].assign(
             self._dataset_recalibration)
-
+        
+        self.datatree['recalibration'].attrs["path_aux_cal_new"] = path_aux_cal_new
+        self.datatree['recalibration'].attrs["path_aux_pp1_new"] = path_aux_pp1_new
+        self.datatree['recalibration'].attrs["path_aux_cal_old"] = path_aux_cal_old
+        self.datatree['recalibration'].attrs["path_aux_pp1_old"] = path_aux_pp1_old
         # return self._dataset
 
     def apply_calibration_and_denoising(self):
